import fs from 'fs';
import { EOL } from 'os';
import acorn from 'acorn';
import { promisify } from 'util';
import { simple } from 'acorn-walk';

/**
 * @module utils
 */
const rm = promisify(fs.rm);
const writeFile = promisify(fs.writeFile);
async function removeFile(path) {
  if (fs.existsSync(path)) {
    await rm(path);
  }
}
function isString(value) {
  return Object.prototype.toString.call(value) === '[object String]';
}
function parse(content) {
  let named = false;
  const styles = [];
  const ast = acorn.parse(content, {
    sourceType: 'module',
    ecmaVersion: 'latest'
  });
  simple(ast, {
    ExpressionStatement(node) {
      simple(node, {
        AssignmentExpression({ left }) {
          if (left.type === 'MemberExpression') {
            const { property } = left;
            if (property.type === 'Identifier') {
              if (property.name === 'locals') {
                simple(node, {
                  Property({ key: left, value: right }) {
                    const { value: key } = left;
                    const { value: value } = right;
                    if (isString(key) && isString(value)) {
                      styles.push([key, value]);
                    }
                  }
                });
              }
            }
          }
        }
      });
    },
    ExportNamedDeclaration(node) {
      named = true;
      simple(node, {
        VariableDeclarator({ id, init }) {
          const { name: key } = id;
          const { value } = init;
          if (isString(key) && isString(value)) {
            styles.push([key, value]);
          }
        }
      });
    }
  });
  return [styles, named];
}
function generateTypings(content, banner, eol = EOL) {
  const [styles, named] = parse(content);
  if (styles.length > 0) {
    const typings = banner ? [banner, ''] : [];
    if (named) {
      for (const [key, value] of styles) {
        typings.push(`export const ${key}: ${JSON.stringify(value)};`);
      }
    } else {
      typings.push('declare const styles: {');
      for (const [key, value] of styles) {
        typings.push(`  ${key}: ${JSON.stringify(value)}`);
      }
      typings.push('};', '', 'export default styles;');
    }
    return typings.join(eol);
  }
  return null;
}

export { generateTypings, isString, parse, removeFile, rm, writeFile };
